/*
  device - VPD support routines for devices.
  
  This file is part of the Linux lsvpd package.
  
  (C) Copyright IBM Corp. 2005
  
  Maintained by  Martin Schwenke <martins@au.ibm.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.
 
  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
  $Id: device_scsi.c,v 1.2 2005/06/14 06:28:45 martins Exp $
*/

/* For asprintf(3). */
#define _GNU_SOURCE

#include <string.h>
#include <stdio.h>

#include "lsvpd.h"
#include "init.h"

#include "device.h"

#include "device_scsi.h"

/********************************************************************/

bool
device_parse_scsi_address(const char * s,
			  device_scsi_address_t * a)
{
	return (4 == sscanf(s,
			    "%u:%u:%u:%u",
			    &a->host, &a->bus, &a->target, &a->lun));
}

bool
device_parse_scsi_bus_info(const device_bus_info_t bus_info,
			   device_scsi_address_t * a)
{
	const char * ba;

	ba = device_find_address(DEVICE_SCSI_TYPE, bus_info);
	return ((NULL != ba) && device_parse_scsi_address(ba, a));
}

/********************************************************************/

static bool
device_scsi_validate (const device_t type,
		      const device_bus_info_t bus_info)
{
	bool ret;
	device_scsi_address_t a;

	ret = true;

	if (device_parse_scsi_bus_info(bus_info, &a) &&
	    (0xff == a.bus) &&
	    (0xff == a.target) &&
	    (0xff == a.lun))
		ret = false;

	return ret;
}

static char *
device_scsi_get_adapter_name (const device_t type,
			      const device_bus_info_t bus_info)
{
	char * ret;
	device_scsi_address_t a;

	ret = NULL;

	if (device_parse_scsi_bus_info(bus_info, &a)) {
		// FIXME: "host" should not be hardcoded.
		if (-1 == asprintf(&ret, "host%d", a.host))
			ret = NULL;
	}

	return ret;
}

static const char * device_scsi_vpd_sort_order[] =
{
	"FC", "DS", "AX", "PL", "MF", "TM", "CD", "LL",
	"YL", "FN", "RL", "SN", "EC", "PN",
	NULL
};

static void
device_scsi_vpd_sort (const device_t type,
		      vpd_list_t ** vpd)
{
	vpd_list_sort(vpd, device_scsi_vpd_sort_order);
}

/********************************************************************/

/*
 * Defaults:
 *   retrieve_vpd returns false.
 */

static void
init(void)
{
	device_functions_t s = 
		{
			.type             = DEVICE_SCSI_TYPE,
			.validate         = device_scsi_validate,
			.get_adapter_name = device_scsi_get_adapter_name,
			.vpd_sort         = device_scsi_vpd_sort,
		};
	device_type_register(&s);
}

INIT(init);
